using System;
using System.IO;
using Microsoft.Win32;

namespace SDIMDIwizardinstaller
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	class Class1
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{

			// installation sequence
			// 1- get VC# install dir (regkey)
			// 2- copy template files in the VC# subfolder for files
			// 3- copy vsz files in the VC# subfolder for project wizards
			// 4- add two entries in the VC# vsdir file so that the IDE sees them

			// check out cmdline (type 2002 for VC#2002, 2003 for VC#2003 none or everything else for VC#2005)
			bool bIsFor2002 = args.GetLength(0)>0 && args[0] == "2002";
			bool bIsFor2003 = args.GetLength(0)>0 && args[0] == "2003";

			// 1- regkey
			RegistryKey k;
			if (bIsFor2002)
				k = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(@"Software\Microsoft\VisualStudio\7.0", true);
			else if (bIsFor2003)
				k = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(@"Software\Microsoft\VisualStudio\7.1", true);
			else
				k = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(@"Software\Microsoft\VisualStudio\8.0", true);
			String szWizardDir = (String) k.GetValue("InstallDir") + @"..\..\VC#\";

			if ( !Directory.Exists(szWizardDir) )
			{
				System.Console.WriteLine("Make sure to install VC# first");
				return;
			}

			// 2- template files
			String szSrcDir = AppDomain.CurrentDomain.BaseDirectory;
			RecurseCopyFiles(szSrcDir + @"\CSharpSDIWiz", szWizardDir + @"VC#Wizards\CSharpSDIWiz");
			RecurseCopyFiles(szSrcDir + @"\CSharpMDIWiz", szWizardDir + @"VC#Wizards\CSharpMDIWiz");

			// 3- vsz files
			if (bIsFor2002) // VC# 2002
			{
				File.Copy(szSrcDir + @"\CSharpSDI_VS2002.vsz", szWizardDir + @"CSharpProjects\CSharpSDI.vsz",true);
				File.Copy(szSrcDir + @"\CSharpMDI_VS2002.vsz", szWizardDir + @"CSharpProjects\CSharpMDI.vsz",true);
			}
			else if (bIsFor2003)// VC# 2003
			{
				File.Copy(szSrcDir + @"\CSharpSDI_VS2003.vsz", szWizardDir + @"CSharpProjects\CSharpSDI.vsz",true);
				File.Copy(szSrcDir + @"\CSharpMDI_VS2003.vsz", szWizardDir + @"CSharpProjects\CSharpMDI.vsz",true);
			}
			else // VC# 2005
			{
				File.Copy(szSrcDir + @"\CSharpSDI_VS2005.vsz", szWizardDir + @"CSharpProjects\CSharpSDI.vsz",true);
				File.Copy(szSrcDir + @"\CSharpMDI_VS2005.vsz", szWizardDir + @"CSharpProjects\CSharpMDI.vsz",true);
			}

			// 4- update vsdir file (append 2 entries if they don't exist yet)
			bool bAlreadyInstalled = false;
			bool bFileExists = false;

			if (File.Exists( szWizardDir + @"CSharpProjects\CSharp.vsdir" ))
			{
				bFileExists = true;

				StreamReader sr = new StreamReader( szWizardDir + @"CSharpProjects\CSharp.vsdir" );
				String line;
				while ((line = sr.ReadLine()) != null) 
				{
					if (line.IndexOf("CSharpSDI.vsz") > -1)
					{
						bAlreadyInstalled = true;
						break;
					}
				}
				sr.Close();
			}

            if (!bFileExists || !bAlreadyInstalled)
			{
				using (StreamWriter sw = File.AppendText( szWizardDir + @"CSharpProjects\CSharp.vsdir" )) 
				{
					sw.WriteLine("CSharpSDI.vsz|{FAE04EC1-301F-11d3-BF4B-00C04F79EFBC}|SDI Application|11|Builds a Windows single document interface (SDI) application|{FAE04EC1-301F-11d3-BF4B-00C04F79EFBC}|4554| |SDIApplication");
					sw.WriteLine("CSharpMDI.vsz|{FAE04EC1-301F-11d3-BF4B-00C04F79EFBC}|MDI Application|12|Builds a Windows multiple document interface (MDI) application|{FAE04EC1-301F-11d3-BF4B-00C04F79EFBC}|4554| |MDIApplication");
					sw.Close();
				}
            }

		}

		static void RecurseCopyFiles(String szSrcDir, String szDestDir)
		{
			if ( !Directory.Exists(szDestDir) )
				Directory.CreateDirectory(szDestDir);

			string [] fileEntries = Directory.GetFiles(szSrcDir);
			foreach(string fileName in fileEntries)
				File.Copy(fileName, szDestDir + fileName.Substring(fileName.LastIndexOf('\\')),true);

			// recurse
			string [] subdirectoryEntries = Directory.GetDirectories(szSrcDir + @"\");
			foreach(string subdirectory in subdirectoryEntries)
			{
				String szSrcNextDir = subdirectory;
				String szDestNextDir = szDestDir + subdirectory.Substring(subdirectory.LastIndexOf('\\'));
				RecurseCopyFiles(szSrcNextDir, szDestNextDir);
			}

		}
	}
}
